//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Common.h"
#include "Moa_AutoPointers.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_common {
//---------------------------------------------------------------------------
const AnsiString TCommon::Title = "Dupy Scan";
const AnsiString TCommon::About = "Dupy Scan 2.0.0\r\n\r\nCopyright (C) 2007-2012 Moochos. All rights reserved.";
const AnsiString TCommon::HelpFile = ExtractFilePath(Application->ExeName) + "Help\\index.html";
const AnsiString TCommon::SettingsFile = ExtractFilePath(Application->ExeName) + "Settings.ini";
//---------------------------------------------------------------------------
AnsiString __fastcall TCommon::FormatFileTime(const FILETIME& AFileTime, bool AToLocal)
{
	if(AFileTime.dwLowDateTime == 0 && AFileTime.dwHighDateTime == 0) {
		return "(Unknown)";
	}
	FILETIME LFileTime = AFileTime;
	SYSTEMTIME LSystemTime = {0};
	if((AToLocal && !::FileTimeToLocalFileTime(&AFileTime, &LFileTime)) ||
		!::FileTimeToSystemTime(&LFileTime, &LSystemTime)) {
		return "(Error)";
	}
	ULARGE_INTEGER LSerialTime = {0};
	LSerialTime.LowPart = LFileTime.dwLowDateTime;
	LSerialTime.HighPart = LFileTime.dwHighDateTime;
	return AnsiString().sprintf(
		"%04u-%02u-%02u %02u:%02u:%02u.%07Lu",
		LSystemTime.wYear,
		LSystemTime.wMonth,
		LSystemTime.wDay,
		LSystemTime.wHour,
		LSystemTime.wMinute,
		LSystemTime.wSecond,
		LSerialTime.QuadPart % (1000 * 1000 * 10));
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommon::FormatInteger(int AInteger)
{
	return TAnsiStringUtilities::FormatInteger(AInteger, 3, ',');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommon::FormatInteger(unsigned int AInteger)
{
	return TAnsiStringUtilities::FormatInteger(AInteger, 3, ',');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommon::FormatInteger(__int64 AInteger)
{
	return TAnsiStringUtilities::FormatInteger(AInteger, 3, ',');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TCommon::FormatInteger(unsigned __int64 AInteger)
{
	return TAnsiStringUtilities::FormatInteger(AInteger, 3, ',');
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TSettingsHandler::TSettingsHandler() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandler::~TSettingsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TSettingsHandlers::Initialize()
{
	try {
		FList = new TStringList();
		static_cast<TStringList*>(FList)->Duplicates = dupError;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandlers::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandlers::TSettingsHandlers() :
	TObject(),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandlers::~TSettingsHandlers()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandlers::Add(TSettingsHandler* AItem, const AnsiString& ASection)
{
	List->AddObject(ASection, AItem);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandlers::LoadSettings()
{
	TAutoPointer<TMemIniFile> LSettings = new TMemIniFile(TCommon::SettingsFile);
	for(int LI = 0; LI < List->Count; ++LI) {
		static_cast<TSettingsHandler*>(List->Objects[LI])->LoadSettings(
			LSettings,
			List->Strings[LI]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandlers::SaveSettings()
{
	TAutoPointer<TMemIniFile> LSettings = new TMemIniFile(TCommon::SettingsFile);
	for(int LI = 0; LI < List->Count; ++LI) {
		static_cast<TSettingsHandler*>(List->Objects[LI])->SaveSettings(
			LSettings,
			List->Strings[LI]);
	}
	LSettings->UpdateFile();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TModel::TModel() :
	TSettingsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TModel::~TModel()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TView::TView() :
	TSettingsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TView::~TView()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TCommands::HandleFeatureUnavailable()
{
	throw EAppError("The feature is unavailable.");
}
//---------------------------------------------------------------------------
__fastcall TCommands::TCommands() :
	TSettingsHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TCommands::~TCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
} // End namespace App_common
//---------------------------------------------------------------------------

