//---------------------------------------------------------------------------
#ifndef App_DocumentH
#define App_DocumentH
//---------------------------------------------------------------------------
#include "App_ScanEntries.h"
namespace Dialogs {class TSaveDialog;} using namespace Dialogs;
//---------------------------------------------------------------------------
namespace App_document {
//---------------------------------------------------------------------------
enum TScanAspect
{
	saDirectory,
	saName,
	saBaseName,
	saExtension,
	saSize,
	saType,
	saCreationTime,
	saLastWriteTime,
	saLastAccessTime,
	saAttributes,
	saHash,
	saBinaries,
};
typedef Set<TScanAspect, saDirectory, saBinaries> TScanAspects;
//---------------------------------------------------------------------------
class TDocument : public TModel
{
private:
	TScanEntries* FEntries;
	TScanAspects FAspects;
	bool FRememberSettings;
	TNotifyEvent FOnAspectsChange;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TScanAspects __fastcall GetAspects();
	void __fastcall SetAspects(const TScanAspects& AAspects);
protected:
	virtual void __fastcall DoAspectsChange();
public:
	__fastcall TDocument();
	virtual __fastcall ~TDocument();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property TScanEntries* Entries = {read=FEntries};
	__property TScanAspects Aspects = {read=GetAspects, write=SetAspects};
	__property bool RememberSettings = {read=FRememberSettings, write=FRememberSettings};
	__property TNotifyEvent OnAspectsChange = {read=FOnAspectsChange, write=FOnAspectsChange};
};
//---------------------------------------------------------------------------
class TDocumentCommands : public TCommands
{
private:
	TDocument* FDocument;
	TSaveDialog* FSaveResultsDialog;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	__property TSaveDialog* SaveResultsDialog = {read=FSaveResultsDialog};
	void __fastcall TaskStatus(TObject* ASender, const AnsiString& AMessage);
	void __fastcall TaskProgress(TObject* ASender, int ACurrent, int AAll);
	void __fastcall TaskTerminate(TObject* ASender);
	void __fastcall TaskException(TObject* ASender, Exception* AException);
	__property TDocument* Document = {read=FDocument};
protected:
public:
	__fastcall TDocumentCommands(TDocument* ADocument);
	virtual __fastcall ~TDocumentCommands();
	virtual void __fastcall Scan();
	virtual bool __fastcall CanScan();
	virtual void __fastcall ShowOptions();
	virtual bool __fastcall CanShowOptions();
	void __fastcall SaveResults();
	bool __fastcall CanSaveResults();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
};
//---------------------------------------------------------------------------
} // End namespace App_document
using namespace App_document;
//---------------------------------------------------------------------------
#endif

