//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Scan.h"
#include "Moa_AutoPointers.h"
#include "Moa_Hash.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_scan {
//---------------------------------------------------------------------------
void __fastcall TScanMatcher::Continue()
{
	if(!DoContinue()) {
		Abort();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanMatcher::DoContinue()
{
	bool LCanContinue = true;
	if(OnContinue != NULL) {
		OnContinue(this, LCanContinue);
	}
	return LCanContinue;
}
//---------------------------------------------------------------------------
__fastcall TScanMatcher::TScanMatcher() :
	TObject(),
	FOnContinue(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanMatcher::~TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanDirectoryMatcher::TScanDirectoryMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanDirectoryMatcher::~TScanDirectoryMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanDirectoryMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return SameFileName(ARecord1.Directory, ARecord2.Directory);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanNameMatcher::TScanNameMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanNameMatcher::~TScanNameMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanNameMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return SameFileName(ARecord1.Name, ARecord2.Name);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanBaseNameMatcher::TScanBaseNameMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanBaseNameMatcher::~TScanBaseNameMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanBaseNameMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return SameFileName(ARecord1.BaseName, ARecord2.BaseName);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanExtensionMatcher::TScanExtensionMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanExtensionMatcher::~TScanExtensionMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanExtensionMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return SameFileName(ARecord1.Extension, ARecord2.Extension);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanSizeMatcher::TScanSizeMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanSizeMatcher::~TScanSizeMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanSizeMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return ARecord1.Size == ARecord2.Size;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanTypeMatcher::TScanTypeMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanTypeMatcher::~TScanTypeMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanTypeMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return AnsiSameStr(ARecord1.Type, ARecord2.Type);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanCreationTimeMatcher::TScanCreationTimeMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanCreationTimeMatcher::~TScanCreationTimeMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanCreationTimeMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return ::CompareFileTime(
		&ARecord1.CreationTime,
		&ARecord2.CreationTime) == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanLastWriteTimeMatcher::TScanLastWriteTimeMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanLastWriteTimeMatcher::~TScanLastWriteTimeMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanLastWriteTimeMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return ::CompareFileTime(
		&ARecord1.LastWriteTime,
		&ARecord2.LastWriteTime) == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanLastAccessTimeMatcher::TScanLastAccessTimeMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanLastAccessTimeMatcher::~TScanLastAccessTimeMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanLastAccessTimeMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return ::CompareFileTime(
		&ARecord1.LastAccessTime,
		&ARecord2.LastAccessTime) == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanAttributesMatcher::TScanAttributesMatcher() :
	TScanMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanAttributesMatcher::~TScanAttributesMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanAttributesMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return ARecord1.Attributes == ARecord2.Attributes;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const int TScanHashMatcher::BufferSize = 1024 * 32;
//---------------------------------------------------------------------------
void __fastcall TScanHashMatcher::Initialize()
{
	try {
	 	FBuffer = new byte[BufferSize];
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanHashMatcher::Uninitialize()
{
	delete FBuffer;
 	FBuffer = NULL;
}
//---------------------------------------------------------------------------
TScanRecord& __fastcall TScanHashMatcher::EnsureHashCalculated(TScanRecord& ARecord)
{
	if(ARecord.Hash.Length() == 0) {
		CalculateHash(ARecord);
	}
 	return ARecord;
}
//---------------------------------------------------------------------------
TScanRecord& __fastcall TScanHashMatcher::CalculateHash(TScanRecord& ARecord)
{
	TAutoPointer<THashFunction> LHashFunction = new TSha256HashFunction();
	try {
		TAutoPointer<TStream> LFileStream = new TFileStream(ARecord.Path, fmOpenRead | fmShareDenyWrite);
		while(true) {
			Continue();
			__int64 LDataLength = LFileStream->Read(Buffer, BufferSize);
			if(LDataLength == 0) {
				break;
			}
			LHashFunction->Update(Buffer, LDataLength);
		}
		ARecord.Hash = LHashFunction->Finish();
	}
	catch(EStreamError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not get the hash value of the file \"%s\".",
			ARecord.Path.c_str()));
	}
	return ARecord;
}
//---------------------------------------------------------------------------
__fastcall TScanHashMatcher::TScanHashMatcher() :
	TScanMatcher(),
	FBuffer(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TScanHashMatcher::~TScanHashMatcher()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TScanHashMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	return EnsureHashCalculated(ARecord1).Hash == EnsureHashCalculated(ARecord2).Hash;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const int TScanBinariesMatcher::BufferSize = 1024 * 32;
//---------------------------------------------------------------------------
void __fastcall TScanBinariesMatcher::Initialize()
{
	try {
	 	FBuffer1 = new byte[BufferSize];
	 	FBuffer2 = new byte[BufferSize];
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanBinariesMatcher::Uninitialize()
{
	delete FBuffer2;
 	FBuffer2 = NULL;

	delete FBuffer1;
 	FBuffer1 = NULL;
}
//---------------------------------------------------------------------------
__fastcall TScanBinariesMatcher::TScanBinariesMatcher() :
	TScanMatcher(),
	FBuffer1(NULL),
	FBuffer2(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanBinariesMatcher::~TScanBinariesMatcher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TScanBinariesMatcher::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	if(ARecord1.Size != ARecord2.Size) {
		return false;
	}
	TScanRecord* LCurrentRecord = NULL;
	try {
		LCurrentRecord = &ARecord1;
		TAutoPointer<TStream> LStream1 = new TFileStream(ARecord1.Path, fmOpenRead | fmShareDenyWrite);
		LStream1->Position = 0;
		LCurrentRecord = &ARecord2;
		TAutoPointer<TStream> LStream2 = new TFileStream(ARecord2.Path, fmOpenRead | fmShareDenyWrite);
		LStream2->Position = 0;
		DWORDLONG LCurrent = 0;
		DWORDLONG LAll = ARecord1.Size;
		while(LCurrent < LAll) {
			Continue();
			int LReadSize = BufferSize;
			if(LAll - LCurrent < LReadSize) {
				LReadSize = LAll - LCurrent;
			}
			LCurrentRecord = &ARecord1;
			LStream1->ReadBuffer(Buffer1, LReadSize);
			LCurrentRecord = &ARecord2;
			LStream2->ReadBuffer(Buffer2, LReadSize);
			if(!CompareMem(Buffer1, Buffer2, LReadSize)) {
				return false;
			}
			LCurrent += LReadSize;
		}
	}
	catch(EFOpenError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not open the file \"%s\".",
			LCurrentRecord->Path.c_str()));
	}
	catch(EReadError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not read the file \"%s\".",
			LCurrentRecord->Path.c_str()));
	}
	return true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TScanMatchers::Initialize()
{
	try {
		FList = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanMatchers::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
__fastcall TScanMatchers::TScanMatchers() :
	TObject(),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TScanMatchers::~TScanMatchers()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TScanMatchers::Add(TScanMatcher* AItem)
{
	List->Add(AItem);
}
//---------------------------------------------------------------------------
bool __fastcall TScanMatchers::Match(TScanRecord& ARecord1, TScanRecord& ARecord2)
{
	for(int LI = 0; LI < List->Count; ++LI) {
		if(!static_cast<TScanMatcher*>(List->Items[LI])->Match(ARecord1, ARecord2)) {
			return false;
		}
	}
	return true;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TScanCommand::ResetResults()
{
	for(int LI = 0; LI < Records.Length; ++LI) {
		if(Records[LI].Group < 0) {
			continue;
		}
		Records[LI].Group = -1;
		DoGroup(LI, Records[LI]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::ScanFiles()
{
	int LCurrent = 0;
	int LAll = Records.Length;
	int LGroup = 1;
	int LTopIndex = 0;
	int LBottomIndex = LAll - 1;
	for(int LIndex1 = 0; LIndex1 <= LBottomIndex; ++LIndex1) {
		Continue();
		TScanRecord& LRecord1 = Records[LIndex1];
		if(LRecord1.Priority != spPrimary || LRecord1.Group >= 0) {
			continue;
		}
		DoMatching(LIndex1, LRecord1);
		for(int LI = LTopIndex + 1; LI <= LBottomIndex; ++LI) {
			if(Records[LI].Group < 0) {
				break;
			}
			++LTopIndex;
		}
		for(int LI = LBottomIndex; LI > LTopIndex; --LI) {
			if(Records[LI].Group < 0) {
				break;
			}
			--LBottomIndex;
		}
		for(int LIndex2 = LTopIndex; LIndex2 <= LBottomIndex; ++LIndex2) {
			if(LIndex2 == LIndex1) {
				continue;
			}
			TScanRecord& LRecord2 = Records[LIndex2];
			if(LRecord2.Group >= 0 || !Matchers->Match(LRecord1, LRecord2)) {
				continue;
			}
			if(LRecord1.Group < 0) {
				LRecord1.Group = LGroup;
				DoGroup(LIndex1, LRecord1);
				DoProgress(++LCurrent, LAll);
			}
			LRecord2.Group = LGroup;
			DoGroup(LIndex2, LRecord2);
			DoProgress(++LCurrent, LAll);
		}
		if(LRecord1.Group < 0) {
			LRecord1.Group = 0;
			DoGroup(LIndex1, LRecord1);
			DoProgress(++LCurrent, LAll);
		}
		else {
			++LGroup;
		}
		DoMatch(LIndex1, LRecord1);
	}
	for(int LI = LTopIndex; LI <= LBottomIndex; ++LI) {
		Continue();
		if(Records[LI].Group >= 0) {
			continue;
		}
		Records[LI].Group = 0;
		DoGroup(LI, Records[LI]);
		DoProgress(++LCurrent, LAll);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::Continue()
{
	if(!DoContinue()) {
		Abort();
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::DoMatching(int AIndex, const TScanRecord& ARecord)
{
	if(OnMatching != NULL) {
		OnMatching(this, AIndex, ARecord);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::DoMatch(int AIndex, const TScanRecord& ARecord)
{
	if(OnMatch != NULL) {
		OnMatch(this, AIndex, ARecord);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::DoGroup(int AIndex, const TScanRecord& ARecord)
{
	if(OnGroup != NULL) {
		OnGroup(this, AIndex, ARecord);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::DoProgress(int ACurrent, int AAll)
{
	if(OnProgress != NULL) {
		OnProgress(this, ACurrent, AAll);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanCommand::DoContinue()
{
	bool LCanContinue = true;
	if(OnContinue != NULL) {
		OnContinue(this, LCanContinue);
	}
	return LCanContinue;
}
//---------------------------------------------------------------------------
__fastcall TScanCommand::TScanCommand(DynamicArray<TScanRecord> ARecords, TScanMatchers* AMatchers) :
	TObject(),
	FRecords(ARecords),
	FMatchers(AMatchers),
	FOnMatching(NULL),
	FOnMatch(NULL),
	FOnGroup(NULL),
	FOnProgress(NULL),
	FOnContinue(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanCommand::~TScanCommand()
{
	delete FMatchers;
	FMatchers = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TScanCommand::Execute()
{
	ResetResults();
	ScanFiles();
}
//---------------------------------------------------------------------------
} // End namespace App_scan
//---------------------------------------------------------------------------

