//---------------------------------------------------------------------------
#ifndef App_ScanH
#define App_ScanH
//---------------------------------------------------------------------------
#include "App_Common.h"
//---------------------------------------------------------------------------
namespace App_scan {
//---------------------------------------------------------------------------
enum TScanPriority
{
	spPrimary,
	spSecondary,
};
//---------------------------------------------------------------------------
struct TScanRecord
{
	AnsiString Path;
	AnsiString Directory;
	AnsiString Name;
	AnsiString BaseName;
	AnsiString Extension;
	DWORDLONG Size;
	AnsiString Type;
	FILETIME CreationTime;
	FILETIME LastWriteTime;
	FILETIME LastAccessTime;
	DWORD Attributes;
	AnsiString Hash;
	TScanPriority Priority;
	int Group;
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TScanContinueEvent)(TObject* ASender, bool& ACanContinue);
//---------------------------------------------------------------------------
class TScanMatcher : public TObject
{
private:
	TScanContinueEvent FOnContinue;
protected:
	virtual void __fastcall Continue();
	virtual bool __fastcall DoContinue();
public:
	__fastcall TScanMatcher();
	virtual __fastcall ~TScanMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2) = 0;
	__property TScanContinueEvent OnContinue = {read=FOnContinue, write=FOnContinue};
};
//---------------------------------------------------------------------------
class TScanDirectoryMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanDirectoryMatcher();
	virtual __fastcall ~TScanDirectoryMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanNameMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanNameMatcher();
	virtual __fastcall ~TScanNameMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanBaseNameMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanBaseNameMatcher();
	virtual __fastcall ~TScanBaseNameMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanExtensionMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanExtensionMatcher();
	virtual __fastcall ~TScanExtensionMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanSizeMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanSizeMatcher();
	virtual __fastcall ~TScanSizeMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanTypeMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanTypeMatcher();
	virtual __fastcall ~TScanTypeMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanCreationTimeMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanCreationTimeMatcher();
	virtual __fastcall ~TScanCreationTimeMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanLastWriteTimeMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanLastWriteTimeMatcher();
	virtual __fastcall ~TScanLastWriteTimeMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanLastAccessTimeMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanLastAccessTimeMatcher();
	virtual __fastcall ~TScanLastAccessTimeMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanAttributesMatcher : public TScanMatcher
{
private:
protected:
public:
	__fastcall TScanAttributesMatcher();
	virtual __fastcall ~TScanAttributesMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanHashMatcher : public TScanMatcher
{
private:
	static const int BufferSize;
	byte* FBuffer;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TScanRecord& __fastcall EnsureHashCalculated(TScanRecord& ARecord);
	TScanRecord& __fastcall CalculateHash(TScanRecord& ARecord);
	__property byte* Buffer = {read=FBuffer};
protected:
public:
	__fastcall TScanHashMatcher();
	virtual __fastcall ~TScanHashMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanBinariesMatcher : public TScanMatcher
{
private:
	static const int BufferSize;
	byte* FBuffer1;
	byte* FBuffer2;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	__property byte* Buffer1 = {read=FBuffer1};
	__property byte* Buffer2 = {read=FBuffer2};
protected:
public:
	__fastcall TScanBinariesMatcher();
	virtual __fastcall ~TScanBinariesMatcher();
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
class TScanMatchers : public TObject
{
private:
	TObjectList* FList;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	__property TObjectList* List = {read=FList};
protected:
public:
	__fastcall TScanMatchers();
	virtual __fastcall ~TScanMatchers();
	virtual void __fastcall Add(TScanMatcher* AItem);
	virtual bool __fastcall Match(TScanRecord& ARecord1, TScanRecord& ARecord2);
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TScanMatchEvent)(TObject* ASender, int AIndex, const TScanRecord& ARecord);
typedef void __fastcall (__closure * TScanGroupEvent)(TObject* ASender, int AIndex, const TScanRecord& ARecord);
typedef void __fastcall (__closure * TScanProgressEvent)(TObject* ASender, int ACurrent, int AAll);
//---------------------------------------------------------------------------
class TScanCommand : public TObject
{
private:
	DynamicArray<TScanRecord> FRecords;
	TScanMatchers* FMatchers;
	TScanMatchEvent FOnMatching;
	TScanMatchEvent FOnMatch;
	TScanGroupEvent FOnGroup;
	TScanProgressEvent FOnProgress;
	TScanContinueEvent FOnContinue;
	void __fastcall ResetResults();
	void __fastcall ScanFiles();
	__property DynamicArray<TScanRecord> Records = {read=FRecords};
	__property TScanMatchers* Matchers = {read=FMatchers};
protected:
	virtual void __fastcall Continue();
	virtual void __fastcall DoMatching(int AIndex, const TScanRecord& ARecord);
	virtual void __fastcall DoMatch(int AIndex, const TScanRecord& ARecord);
	virtual void __fastcall DoGroup(int AIndex, const TScanRecord& ARecord);
	virtual void __fastcall DoProgress(int ACurrent, int AAll);
	virtual bool __fastcall DoContinue();
public:
	__fastcall TScanCommand(DynamicArray<TScanRecord> ARecords, TScanMatchers* AMatchers);
	virtual __fastcall ~TScanCommand();
	virtual void __fastcall Execute();
	__property TScanMatchEvent OnMatching = {read=FOnMatching, write=FOnMatching};
	__property TScanMatchEvent OnMatch = {read=FOnMatch, write=FOnMatch};
	__property TScanGroupEvent OnGroup = {read=FOnGroup, write=FOnGroup};
	__property TScanProgressEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TScanContinueEvent OnContinue = {read=FOnContinue, write=FOnContinue};
};
//---------------------------------------------------------------------------
} // End namespace App_scan
using namespace App_scan;
//---------------------------------------------------------------------------
#endif

