//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <ComCtrls.hpp>
#include <Dialogs.hpp>
#include <IniFiles.hpp>
#include "App_ScanEntries.h"
#include "Moa_AutoPointers.h"
#include "Moa_Clipboard.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Files.h"
#include "Moa_ListViews.h"
#include "Moa_ShellFiles.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_scanentries {
//---------------------------------------------------------------------------
const DWORD TScanEntry::AttributeMask =
	FILE_ATTRIBUTE_ARCHIVE |
	FILE_ATTRIBUTE_HIDDEN |
	FILE_ATTRIBUTE_READONLY |
	FILE_ATTRIBUTE_SYSTEM;
//---------------------------------------------------------------------------
TFileTypeCache* __fastcall TScanEntry::GetFileTypeCache()
{
	static TFileTypeCache* LFileTypeCache = NULL;
	if(LFileTypeCache == NULL) {
		LFileTypeCache = new TFileTypeCache();
	}
	return LFileTypeCache;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntry::Initialize(const AnsiString& AFile, const WIN32_FIND_DATA& AFileData)
{
	try {
		FPath = AFile;
		FDirectory = ExtractFileDir(FPath);
		FName = ExtractFileName(FPath);
		FBaseName = ChangeFileExt(FName, AnsiString());
		FExtension = ExtractFileExt(FPath);
		FExtension = TPathUtilities::StartWithoutCharacter(FExtension, '.');
		FExtension = AnsiUpperCaseFileName(FExtension);
		FSize = TFileDataUtilities::GetSize(AFileData);
		FType = GetFileTypeCache()->DetermineAsRegularFile(FPath);
		FCreationTime = TFileDataUtilities::GetCreationTime(AFileData);
		FLastWriteTime = TFileDataUtilities::GetLastWriteTime(AFileData);
		FLastAccessTime = TFileDataUtilities::GetLastAccessTime(AFileData);
		FAttributes = TFileDataUtilities::GetAttributes(AFileData) & AttributeMask;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntry::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TScanEntry::SetGroup(int AGroup)
{
	int LGroup = AGroup;
	if(LGroup < 0) {
		LGroup = -1;
	}
	FGroup = LGroup;
}
//---------------------------------------------------------------------------
__fastcall TScanEntry::TScanEntry(const AnsiString& AFile, const WIN32_FIND_DATA& AFileData) :
	TObject(),
	FPath(),
	FDirectory(),
	FName(),
	FBaseName(),
	FExtension(),
	FSize(0),
	FType(),
	FCreationTime(),
	FLastWriteTime(),
	FLastAccessTime(),
	FAttributes(0),
	FHash(),
	FPriority(spSecondary),
	FGroup(-1)
{
	Initialize(AFile, AFileData);
}
//---------------------------------------------------------------------------
__fastcall TScanEntry::~TScanEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TScanRecord __fastcall TScanEntry::ToRecord()
{
	TScanRecord LRecord;
	LRecord.Path = Path;
	LRecord.Directory = Directory;
	LRecord.Name = Name;
	LRecord.BaseName = BaseName;
	LRecord.Extension = Extension;
	LRecord.Size = Size;
	LRecord.Type = Type;
	LRecord.CreationTime = CreationTime;
	LRecord.LastWriteTime = LastWriteTime;
	LRecord.LastAccessTime = LastAccessTime;
	LRecord.Attributes = Attributes;
	LRecord.Hash = Hash;
	LRecord.Priority = Priority;
	LRecord.Group = Group;
	return LRecord;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TScanEntryProducer::Initialize(const TStringDynArray& AFiles)
{
	try {
		FFiles = new TStringList();
		FFiles->Capacity = AFiles.Length;
		for(int LI = 0; LI < AFiles.Length; ++LI) {
			FFiles->Add(AFiles[LI]);
		}
		FDirectories = new TStringList();
		FHistory = new TPathList();
		static_cast<TPathList*>(FHistory)->Capacity = AFiles.Length;
		static_cast<TPathList*>(FHistory)->Duplicates = dupIgnore;
		static_cast<TPathList*>(FHistory)->Sorted = true;
		FList = new TObjectList();
		FList->Capacity = AFiles.Length;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntryProducer::Uninitialize()
{
	delete FList;
	FList = NULL;

	delete FHistory;
	FHistory = NULL;

	delete FDirectories;
	FDirectories = NULL;

	delete FFiles;
	FFiles = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntryProducer::ProduceFromFile(const AnsiString& AFile)
{
	try {
		AnsiString LFile = AFile;
		LFile = TPathUtilities::AssureAbsolute(LFile);
		LFile = TPathUtilities::AssureNotPattern(LFile);
		LFile = TFileUtilities::GetLongPath(LFile);
		LFile = ExpandFileName(LFile);
		if(!TStringsUtilities::TryAdd(History, LFile)) {
			return;
		}
		if(DirectoryExists(LFile)) {
			Directories->Add(LFile);
			return;
		}
		TAutoPointer<TScanEntry> LItem = new TScanEntry(LFile, TFileUtilities::GetData(LFile));
		List->Add(LItem);
		LItem.Release();
	}
	catch(EFileError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program cannot accept the file \"%s\".",
			AFile.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntryProducer::ProduceFromDirectory(const AnsiString& ADirectory)
{
	AnsiString LDirectory = IncludeTrailingPathDelimiter(ADirectory);
	TAutoPointer<TFileEnumerator> LEnumerator = new TFileEnumerator(LDirectory + "*");
	if(!LEnumerator->MoveNext()) {
		throw EAppError(AnsiString().sprintf(
			"The program cannot find the file \"%s\".",
			ADirectory.c_str()));
	}
	do {
		if(lstrcmp(LEnumerator->Current.cFileName, ".") != 0 && lstrcmp(LEnumerator->Current.cFileName, "..") != 0) {
			AnsiString LFile = LDirectory + LEnumerator->Current.cFileName;
			if(TStringsUtilities::TryAdd(History, LFile)) {
				if((LEnumerator->Current.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0) {
					Directories->Add(LFile);
				}
				else {
					TAutoPointer<TScanEntry> LItem = new TScanEntry(LFile, LEnumerator->Current);
					List->Add(LItem);
					LItem.Release();
				}
			}
		}
	} while(LEnumerator->MoveNext());
}
//---------------------------------------------------------------------------
DynamicArray<TScanEntry*> __fastcall TScanEntryProducer::GetValues()
{
	DynamicArray<TScanEntry*> LValues;
	LValues.Length = List->Count;
	for(int LI = 0; LI < LValues.Length; ++LI) {
		LValues[LI] = static_cast<TScanEntry*>(List->Items[LI]);
	}
	return LValues;
}
//---------------------------------------------------------------------------
__fastcall TScanEntryProducer::TScanEntryProducer(const TStringDynArray& AFiles) :
	TObject(),
	FFiles(NULL),
	FDirectories(NULL),
	FHistory(NULL),
	FList(NULL)
{
	Initialize(AFiles);
}
//---------------------------------------------------------------------------
__fastcall TScanEntryProducer::~TScanEntryProducer()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntryProducer::Produce()
{
	if(Files->Count > 0) {
		ProduceFromFile(Files->Strings[0]);
		Files->Delete(0);
		return true;
	}
	else if(Directories->Count > 0) {
		ProduceFromDirectory(Directories->Strings[0]);
		Directories->Delete(0);
		return true;
	}
	else {
		return false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntryProducer::Release()
{
	bool LOwnsObjects = List->OwnsObjects;
	List->OwnsObjects = false;
	try {
		List->Clear();
	}
	catch(...) {
		List->OwnsObjects = LOwnsObjects;
		throw;
	}
	List->OwnsObjects = LOwnsObjects;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int TScanEntryDescriber::ImageIndex_Primary = -1;
int TScanEntryDescriber::ImageIndex_Secondary = -1;
int TScanEntryDescriber::StateIndex_NoResult = -1;
int TScanEntryDescriber::StateIndex_Unique = -1;
int TScanEntryDescriber::StateIndex_Duplicate = -1;
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::EvaluatePriority(TScanPriority APriority)
{
	switch(APriority) {
	case spPrimary:
		return 0;
	case spSecondary:
		return 1;
	default:
		throw EAppError("The program cannot accept the priority.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::EvaluateGroup(int AGroup)
{
	if(AGroup < 0) {
		return MaxInt;
	}
	else if(AGroup == 0) {
		return MaxInt - 1;
	}
	else {
		return AGroup;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByPath(TScanEntry* AEntry)
{
	return AEntry->Path;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByDirectory(TScanEntry* AEntry)
{
	return AEntry->Directory;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByName(TScanEntry* AEntry)
{
	return AEntry->Name;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByBaseName(TScanEntry* AEntry)
{
	return AEntry->BaseName;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByExtension(TScanEntry* AEntry)
{
	return AEntry->Extension;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeBySize(TScanEntry* AEntry)
{
	return TCommon::FormatInteger(AEntry->Size);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByType(TScanEntry* AEntry)
{
	return AEntry->Type;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByCreationTime(TScanEntry* AEntry)
{
	return TCommon::FormatFileTime(AEntry->CreationTime, true);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByLastWriteTime(TScanEntry* AEntry)
{
	return TCommon::FormatFileTime(AEntry->LastWriteTime, true);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByLastAccessTime(TScanEntry* AEntry)
{
	return TCommon::FormatFileTime(AEntry->LastAccessTime, true);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByAttributes(TScanEntry* AEntry)
{
	char LBuffer[5];
	char* LPosition = LBuffer;
	*(LPosition++) = (AEntry->Attributes & FILE_ATTRIBUTE_ARCHIVE) != 0 ? 'A' : '-';
	*(LPosition++) = (AEntry->Attributes & FILE_ATTRIBUTE_HIDDEN) != 0 ? 'H' : '-';
	*(LPosition++) = (AEntry->Attributes & FILE_ATTRIBUTE_READONLY) != 0 ? 'R' : '-';
	*(LPosition++) = (AEntry->Attributes & FILE_ATTRIBUTE_SYSTEM) != 0 ? 'S' : '-';
	*LPosition = '\0';
	return AnsiString(LBuffer);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByHash(TScanEntry* AEntry)
{
	return AEntry->Hash;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByPriority(TScanEntry* AEntry)
{
	switch(AEntry->Priority) {
	case spPrimary:
		return "Primary";
	case spSecondary:
		return "Secondary";
	default:
		throw EAppError("The program cannot accept the priority.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanEntryDescriber::DescribeByGroup(TScanEntry* AEntry)
{
	if(AEntry->Group < 0) {
		return AnsiString();
	}
	else if(AEntry->Group == 0) {
		return "(Unique)";
	}
	else {
		return AEntry->Group;
	}
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::DetermineImageIndex(TScanEntry* AEntry)
{
	switch(AEntry->Priority) {
	case spPrimary:
		return ImageIndex_Primary;
	case spSecondary:
		return ImageIndex_Secondary;
	default:
		throw EAppError("The program cannot accept the priority.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::DetermineStateIndex(TScanEntry* AEntry)
{
	if(AEntry->Group < 0) {
		return StateIndex_NoResult;
	}
	else if(AEntry->Group == 0) {
		return StateIndex_Unique;
	}
	else {
		return StateIndex_Duplicate;
	}
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByPath(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareFileName(AEntry1->Path, AEntry2->Path);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByDirectory(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareFileName(AEntry1->Directory, AEntry2->Directory);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByName(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareFileName(AEntry1->Name, AEntry2->Name);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByBaseName(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareFileName(AEntry1->BaseName, AEntry2->BaseName);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByExtension(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareFileName(AEntry1->Extension, AEntry2->Extension);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareBySize(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	if(AEntry1->Size < AEntry2->Size) {
		return -1;
	}
	else if(AEntry1->Size > AEntry2->Size) {
		return 1;
	}
	else {
		return 0;
	}
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByType(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return AnsiCompareStr(AEntry1->Type, AEntry2->Type);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByCreationTime(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	FILETIME LFileTime1 = AEntry1->CreationTime;
	FILETIME LFileTime2 = AEntry2->CreationTime;
	return ::CompareFileTime(&LFileTime1, &LFileTime2);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByLastWriteTime(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	FILETIME LFileTime1 = AEntry1->LastWriteTime;
	FILETIME LFileTime2 = AEntry2->LastWriteTime;
	return ::CompareFileTime(&LFileTime1, &LFileTime2);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByLastAccessTime(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	FILETIME LFileTime1 = AEntry1->LastAccessTime;
	FILETIME LFileTime2 = AEntry2->LastAccessTime;
	return ::CompareFileTime(&LFileTime1, &LFileTime2);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByAttributes(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	DWORD LAttributes1 = AEntry1->Attributes;
	DWORD LAttributes2 = AEntry2->Attributes;
	if(((LAttributes1 ^ LAttributes2) & FILE_ATTRIBUTE_ARCHIVE) != 0) {
		return ((LAttributes1 & FILE_ATTRIBUTE_ARCHIVE) != 0) ? 1 : -1;
	}
	if(((LAttributes1 ^ LAttributes2) & FILE_ATTRIBUTE_HIDDEN) != 0) {
		return ((LAttributes1 & FILE_ATTRIBUTE_HIDDEN) != 0) ? 1 : -1;
	}
	if(((LAttributes1 ^ LAttributes2) & FILE_ATTRIBUTE_READONLY) != 0) {
		return ((LAttributes1 & FILE_ATTRIBUTE_READONLY) != 0) ? 1 : -1;
	}
	if(((LAttributes1 ^ LAttributes2) & FILE_ATTRIBUTE_SYSTEM) != 0) {
		return ((LAttributes1 & FILE_ATTRIBUTE_SYSTEM) != 0) ? 1 : -1;
	}
	return 0;
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByHash(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	if(AEntry1->Hash.Length() == 0) {
		if(AEntry2->Hash.Length() == 0) {
			return 0;
		}
		return 1;
	}
	else if(AEntry2->Hash.Length() == 0) {
		return -1;
	}
	return AnsiCompareText(AEntry1->Hash, AEntry2->Hash);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByPriority(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return EvaluatePriority(AEntry1->Priority) - EvaluatePriority(AEntry2->Priority);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntryDescriber::CompareByGroup(TScanEntry* AEntry1, TScanEntry* AEntry2)
{
	return EvaluateGroup(AEntry1->Group) - EvaluateGroup(AEntry2->Group);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByPath(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByPath(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByDirectory(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByDirectory(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByName(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByName(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByBaseName(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByBaseName(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByExtension(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByExtension(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareBySize(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareBySize(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByType(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByType(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByCreationTime(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByCreationTime(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByLastWriteTime(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByLastWriteTime(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByLastAccessTime(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByLastAccessTime(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByAttributes(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByAttributes(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByHash(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByHash(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByPriority(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByPriority(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::SortCompareByGroup(int AIndex1, int AIndex2, void* AData)
{
	return TScanEntryDescriber::CompareByGroup(
		static_cast<TScanEntries*>(AData)->Items[AIndex1],
		static_cast<TScanEntries*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::Initialize()
{
	try {
		FList = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TScanEntry* __fastcall TScanEntries::GetItem(int AIndex)
{
	return static_cast<TScanEntry*>(List->Items[AIndex]);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DoInsertItems(const TIntegerDynArray& AIndexes)
{
	if(OnInsertItems != NULL) {
		OnInsertItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DoDeleteItems(const TIntegerDynArray& AIndexes)
{
	if(OnDeleteItems != NULL) {
		OnDeleteItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DoReorder(const TListReorderRecords& ARecords)
{
	if(OnReorder != NULL) {
		OnReorder(this, ARecords);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DoItemChange(int AIndex)
{
	if(OnItemChange != NULL) {
		OnItemChange(this, AIndex);
	}
}
//---------------------------------------------------------------------------
__fastcall TScanEntries::TScanEntries() :
	TModel(),
	FList(NULL),
	FOnInsertItems(NULL),
	FOnDeleteItems(NULL),
	FOnClear(NULL),
	FOnReorder(NULL),
	FOnItemChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TScanEntries::~TScanEntries()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::AddItems(const DynamicArray<TScanEntry*>& AItems)
{
	if(AItems.Length == 0) {
		return;
	}
	DeleteItems(FindAllContained(AItems));
	if(List->Capacity < List->Count + AItems.Length) {
		List->Capacity = List->Count + AItems.Length;
	}
	TIntegerDynArray LIndexes;
	LIndexes.Length = AItems.Length;
	for(int LI = 0; LI < AItems.Length; ++LI) {
		LIndexes[LI] = List->Add(AItems[LI]);
	}
	DoInsertItems(LIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		List->Delete(AIndexes[AIndexes.High - LI]);
	}
	DoDeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::Clear()
{
	if(Count == 0) {
		return;
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByPath()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByPath, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByDirectory()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByDirectory, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByName()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByName, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByBaseName()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByBaseName, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByExtension()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByExtension, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortBySize()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareBySize, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByType()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByType, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByCreationTime()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByCreationTime, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByLastWriteTime()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByLastWriteTime, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByLastAccessTime()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByLastAccessTime, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByAttributes()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByAttributes, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByHash()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByHash, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByPriority()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByPriority, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::SortByGroup()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByGroup, this));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::ReverseOrder()
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::ReverseOrder(Count));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::MoveUp(const TIntegerDynArray& AIndexes)
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Slide(Count, AIndexes, -1));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::MoveDown(const TIntegerDynArray& AIndexes)
{
	if(Count == 0) {
		return;
	}
	Reorder(TListAlgorithms::Slide(Count, AIndexes, 1));
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::Reorder(const TListReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	TObjectListUtilities::Reorder(List, ARecords);
	DoReorder(ARecords);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::Find(TScanEntry* AItem)
{
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI] == AItem) {
			return LI;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TScanEntries::FindAllContained(const DynamicArray<TScanEntry*>& AItems)
{
	if(AItems.Length == 0) {
		return TIntegerDynArray();
	}
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder;
	{
		TAutoPointer<TPathList> LTable = new TPathList();
		LTable->Capacity = AItems.Length;
		for(int LI = 0; LI < AItems.Length; ++LI) {
			LTable->Add(AItems[LI]->Path);
		}
		LTable->Sorted = true;
		LBuilder = new TDynamicArrayBuilder<int>();
		LBuilder->Capacity = AItems.Length;
		for(int LI = 0; LI < Count; ++LI) {
			int LIndex;
			if(!LTable->Find(Items[LI]->Path, LIndex)) {
				continue;
			}
			LBuilder->Add(LI);
			LTable->Delete(LIndex);
			if(LTable->Count == 0) {
				break;
			}
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TScanEntries::FindAllDuplicate()
{
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = CountDuplicate();
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Group > 0) {
			LBuilder->Add(LI);
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TScanEntries::FindAllUnique()
{
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = CountUnique();
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Group == 0) {
			LBuilder->Add(LI);
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::CountDuplicate()
{
	return Count - CountByGroup(-1) - CountByGroup(0);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::CountUnique()
{
	return CountByGroup(0);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::CountGroups()
{
	TBooleanDynArray LTable;
	{
		int LMaxGroup = 0;
		for(int LI = 0; LI < Count; ++LI) {
			if(Items[LI]->Group > LMaxGroup) {
				LMaxGroup = Items[LI]->Group;
			}
		}
		LTable.Length = LMaxGroup + 1;
		for(int LI = 0; LI < LTable.Length; ++LI) {
			LTable[LI] = false;
		}
		for(int LI = 0; LI < Count; ++LI) {
			if(Items[LI]->Group > 0) {
				LTable[Items[LI]->Group] = true;
			}
		}
	}
	int LCount = 0;
	for(int LI = 0; LI < LTable.Length; ++LI) {
		if(LTable[LI]) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::CountByPriority(TScanPriority APriority)
{
	int LCount = 0;
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Priority == APriority) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------
int __fastcall TScanEntries::CountByGroup(int AGroup)
{
	int LGroup = AGroup;
	if(LGroup < 0) {
		LGroup = -1;
	}
	int LCount = 0;
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Group == LGroup) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntries::ItemChange(int AIndex)
{
	DoItemChange(AIndex);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TScanEntriesView::TScanEntriesView(TScanEntries* AEntries, TListView* AListView) :
	TView(),
	FEntries(AEntries),
	FListView(AListView)
{
	Entries->OnInsertItems = HandleInsertItems;
	Entries->OnDeleteItems = HandleDeleteItems;
	Entries->OnClear = HandleClear;
	Entries->OnReorder = HandleReorder;
	Entries->OnItemChange = HandleItemChange;
}
//---------------------------------------------------------------------------
__fastcall TScanEntriesView::~TScanEntriesView()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesView::HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::InsertItems(ListView, AIndexes);
		TListViewItemsUtilities::SetSelectionIndexes(ListView, AIndexes);
		TListViewItemUtilities::SetFocused(ListView, AIndexes[AIndexes.High], true);
		TListViewItemUtilities::EnsureVisible(ListView, AIndexes[AIndexes.High], false);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesView::HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::DeleteItems(ListView, AIndexes);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesView::HandleClear(TObject* ASender)
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->Items->Count = 0;
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesView::HandleReorder(TObject* ASender, const TListReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::Reorder(ListView, ARecords);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesView::HandleItemChange(TObject* ASender, int AIndex)
{
	ListView->UpdateItems(AIndex, AIndex);
	ListView->Update();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TScanEntriesSelection::GetItem(int AIndex)
{
	return TListViewItemUtilities::IsSelected(ListView, AIndex);
}
//---------------------------------------------------------------------------
int __fastcall TScanEntriesSelection::GetCount()
{
	return ListView->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TScanEntriesSelection::GetSelectionCount()
{
	return ListView->SelCount;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TScanEntriesSelection::GetSelectionIndexes()
{
	return TListViewItemsUtilities::GetSelectionIndexes(ListView);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesSelection::SetSelectionIndexes(const TIntegerDynArray& AIndexes)
{
	TListViewItemsUtilities::SetSelectionIndexes(ListView, AIndexes);
}
//---------------------------------------------------------------------------
__fastcall TScanEntriesSelection::TScanEntriesSelection(TScanEntries* AEntries, TListView* AListView) :
	TModel(),
	FEntries(AEntries),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanEntriesSelection::~TScanEntriesSelection()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesSelection::SelectAll()
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesSelection::ReverseSelection()
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::ReverseSelection(ListView);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Initialize()
{
	try {
		FClipboardWatcher = new TClipboardWatcher();
		FClipboardWatcher->Enabled = true;
		FClipboardWatcher->OnChange = ClipboardWatcherChange;
		FAddRegularFilesDialog = new TOpenDialog(NULL);
		FAddRegularFilesDialog->Title = "Add Regular Files";
		FAddRegularFilesDialog->Filter = "All (*.*)|*.*";
		FAddRegularFilesDialog->FilterIndex = 1;
		FAddRegularFilesDialog->Options = TOpenOptions(FAddRegularFilesDialog->Options)
			<< ofAllowMultiSelect
			<< ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Uninitialize()
{
	delete FAddRegularFilesDialog;
	FAddRegularFilesDialog = NULL;

	delete FClipboardWatcher;
	FClipboardWatcher = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::AddFiles(const TStringDynArray& AFiles)
{
	if(AFiles.Length == 0) {
		return;
	}
	TAutoPointer<TScanEntryProducer> LProducer = new TScanEntryProducer(AFiles);
	while(LProducer->Produce()) {}
	Entries->AddItems(LProducer->Values);
	LProducer->Release();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Delete(const TIntegerDynArray& AIndexes)
{
	Entries->DeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SetPriorityTo(TScanPriority APriority)
{
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		if(Entries->Items[LIndexes[LI]]->Priority == APriority) {
			continue;
		}
		Entries->Items[LIndexes[LI]]->Priority = APriority;
		Entries->ItemChange(LIndexes[LI]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ClipboardWatcherChange(TObject* ASender)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}                  
}
//---------------------------------------------------------------------------
__fastcall TScanEntriesCommands::TScanEntriesCommands(TScanEntries* AEntries, TScanEntriesSelection* AEntriesSelection) :
	TCommands(),
	FEntries(AEntries),
	FEntriesSelection(AEntriesSelection),
	FClipboardWatcher(NULL),
	FAddRegularFilesDialog(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TScanEntriesCommands::~TScanEntriesCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::New()
{
	if(!CanNew()) {
		HandleFeatureUnavailable();
	}
	if(Entries->Count == 0) {
		return;
	}
	if(Entries->Count != Entries->CountUnique() && Application->MessageBox(
		"Do you want to remove all the files from the list?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	Entries->Clear();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanNew()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::AddRegularFiles()
{
	if(!CanAddRegularFiles()) {
		HandleFeatureUnavailable();
	}
	AddRegularFilesDialog->FileName = AnsiString();
	if(!AddRegularFilesDialog->Execute()) {
		Abort();
	}
	AddFiles(TAnsiStringDynamicArrayUtilities::FromList(AddRegularFilesDialog->Files));
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanAddRegularFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::AddDirectoryEntries()
{
	if(!CanAddDirectoryEntries()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = GetCurrentDir();
	if(!SelectDirectory("Choose a directory.", WideString(), LFile)) {
		Abort();
	}
	SetCurrentDir(LFile);
	AddFiles(TAnsiStringDynamicArrayUtilities::FromElements(LFile));
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanAddDirectoryEntries()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Cut()
{
	if(!CanCut()) {
		HandleFeatureUnavailable();
	}
	Copy();
	Delete(EntriesSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanCut()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Copy()
{
	if(!CanCopy()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TStrings> LLines;
	{
		TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
		LLines = new TStringList();
		LLines->Capacity = LIndexes.Length;
		for(int LI = 0; LI < LIndexes.Length; ++LI) {
			LLines->Add(Entries->Items[LIndexes[LI]]->Path);
		}
	}
	Clipboard()->AsText = LLines->Text;
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanCopy()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Paste()
{
	if(!CanPaste()) {
		HandleFeatureUnavailable();
	}
	TStringDynArray LFiles;
	LFiles = TAnsiStringDynamicArrayUtilities::FromText(Clipboard()->AsText);
	LFiles = TAnsiStringDynamicArrayUtilities::RemoveBlankLines(LFiles);
	AddFiles(LFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanPaste()
{
	return Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Delete()
{
	if(!CanDelete()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	if(Application->MessageBox(
		AnsiString().sprintf(
			"Do you want to remove the %s files from the list?",
			TCommon::FormatInteger(LIndexes.Length).c_str()).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	Delete(LIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanDelete()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SelectAll()
{
	if(!CanSelectAll()) {
		HandleFeatureUnavailable();
	}
	EntriesSelection->SelectAll();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSelectAll()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ReverseSelection()
{
	if(!CanReverseSelection()) {
		HandleFeatureUnavailable();
	}
	EntriesSelection->ReverseSelection();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanReverseSelection()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SelectFirstInGroup()
{
	if(!CanSelectFirstInGroup()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LTable;
	{
		int LMaxGroup = 0;
		for(int LI = 0; LI < Entries->Count; ++LI) {
			if(Entries->Items[LI]->Group > LMaxGroup) {
				LMaxGroup = Entries->Items[LI]->Group;
			}
		}
		LTable.Length = LMaxGroup + 1;
		for(int LI = 0; LI < LTable.Length; ++LI) {
			LTable[LI] = -1;
		}
		for(int LI = 0; LI < Entries->Count; ++LI) {
			if(Entries->Items[LI]->Group > 0 && LTable[Entries->Items[LI]->Group] < 0) {
				LTable[Entries->Items[LI]->Group] = LI;
			}
		}
	}
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = LTable.Length;
	for(int LI = 0; LI < LTable.Length; ++LI) {
		if(LTable[LI] >= 0) {
			LBuilder->Add(LTable[LI]);
		}
	}
	EntriesSelection->SelectionIndexes = LBuilder->Finish();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSelectFirstInGroup()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByPath()
{
	if(!CanSortByPath()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByPath();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByPath()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByDirectory()
{
	if(!CanSortByDirectory()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByDirectory();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByDirectory()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByName()
{
	if(!CanSortByName()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByName();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByName()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByBaseName()
{
	if(!CanSortByBaseName()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByBaseName();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByBaseName()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByExtension()
{
	if(!CanSortByExtension()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByExtension();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByExtension()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortBySize()
{
	if(!CanSortBySize()) {
		HandleFeatureUnavailable();
	}
	Entries->SortBySize();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortBySize()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByType()
{
	if(!CanSortByType()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByType();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByType()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByCreationTime()
{
	if(!CanSortByCreationTime()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByCreationTime();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByCreationTime()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByLastWriteTime()
{
	if(!CanSortByLastWriteTime()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByLastWriteTime();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByLastWriteTime()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByLastAccessTime()
{
	if(!CanSortByLastAccessTime()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByLastAccessTime();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByLastAccessTime()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByAttributes()
{
	if(!CanSortByAttributes()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByAttributes();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByAttributes()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByHash()
{
	if(!CanSortByHash()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByHash();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByHash()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByPriority()
{
	if(!CanSortByPriority()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByPriority();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByPriority()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SortByGroup()
{
	if(!CanSortByGroup()) {
		HandleFeatureUnavailable();
	}
	Entries->SortByGroup();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSortByGroup()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ReverseOrder()
{
	if(!CanReverseOrder()) {
		HandleFeatureUnavailable();
	}
	Entries->ReverseOrder();
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanReverseOrder()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::MoveUp()
{
	if(!CanMoveUp()) {
		HandleFeatureUnavailable();
	}
	Entries->MoveUp(EntriesSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanMoveUp()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::MoveDown()
{
	if(!CanMoveDown()) {
		HandleFeatureUnavailable();
	}
	Entries->MoveDown(EntriesSelection->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanMoveDown()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	if(LIndexes.Length > 1 && Application->MessageBox(
		AnsiString().sprintf(
			"Do you want to open %s files?",
			TCommon::FormatInteger(LIndexes.Length).c_str()).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		AnsiString LFile = Entries->Items[LIndexes[LI]]->Path;
		AnsiString LDirectory = ExtractFileDir(LFile);
		SHELLEXECUTEINFO LExecuteInfo = {0};
		LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
		LExecuteInfo.lpFile = LFile.c_str();
		LExecuteInfo.lpDirectory = LDirectory.c_str();
		LExecuteInfo.nShow = SW_SHOWNORMAL;
		if(!ShellExecuteEx(&LExecuteInfo)) {
			throw EAppError(AnsiString().sprintf(
				"The program could not open the file \"%s\".",
				LFile.c_str()));
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanOpen()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ExploreDirectory()
{
	if(!CanExploreDirectory()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	if(LIndexes.Length > 1 && Application->MessageBox(
		AnsiString().sprintf(
			"Do you want to explore %s directories?",
			TCommon::FormatInteger(LIndexes.Length).c_str()).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		AnsiString LFile = Entries->Items[LIndexes[LI]]->Directory;
		AnsiString LDirectory = LFile;
		SHELLEXECUTEINFO LExecuteInfo = {0};
		LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI;
		LExecuteInfo.lpVerb = "explore";
		LExecuteInfo.lpFile = LFile.c_str();
		LExecuteInfo.lpDirectory = LDirectory.c_str();
		LExecuteInfo.nShow = SW_SHOWNORMAL;
		if(!ShellExecuteEx(&LExecuteInfo)) {
			throw EAppError(AnsiString().sprintf(
				"The program could not explore the directory \"%s\".",
				LFile.c_str()));
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanExploreDirectory()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::MoveToRecycleBin()
{
	if(!CanMoveToRecycleBin()) {
		HandleFeatureUnavailable();
	}

	// Move files to the recycle bin
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	if(Application->MessageBox(
		AnsiString().sprintf(
			"Do you want to move %s files to the recycle bin?",
			TCommon::FormatInteger(LIndexes.Length).c_str()).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	TStringDynArray LPaths;
	LPaths.Length = LIndexes.Length;
	for(int LI = 0; LI < LPaths.Length; ++LI) {
		LPaths[LI] = Entries->Items[LIndexes[LI]]->Path;
	}
	bool LCompleted = TShellFileUtilities::Delete(LPaths, true);

	// Update the entries
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = LIndexes.Length;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		if(!TFileUtilities::Exists(Entries->Items[LIndexes[LI]]->Path)) {
			LBuilder->Add(LIndexes[LI]);
		}
	}
	Delete(LBuilder->Finish());

	if(!LCompleted) {
		Abort();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanMoveToRecycleBin()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ShowTotalSize()
{
	if(!CanShowTotalSize()) {
		HandleFeatureUnavailable();
	}
	DWORDLONG LTotalSize = 0;
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		LTotalSize += Entries->Items[LIndexes[LI]]->Size;
	}
	Application->MessageBox(
		AnsiString().sprintf(
			"The total size of the %s files is %s bytes.",
			TCommon::FormatInteger(LIndexes.Length).c_str(),
			TCommon::FormatInteger(LTotalSize).c_str()).c_str(),
		Application->Title.c_str(),
		MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanShowTotalSize()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::ShowProperties()
{
	if(!CanShowProperties()) {
		HandleFeatureUnavailable();
	}
	TIntegerDynArray LIndexes = EntriesSelection->SelectionIndexes;
	if(LIndexes.Length > 1 && Application->MessageBox(
		AnsiString().sprintf(
			"Do you want to show the properties of %s files?",
			TCommon::FormatInteger(LIndexes.Length).c_str()).c_str(),
		Application->Title.c_str(),
		MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
		Abort();
	}
	for(int LI = 0; LI < LIndexes.Length; ++LI) {
		AnsiString LFile = Entries->Items[LIndexes[LI]]->Path;
		SHELLEXECUTEINFO LExecuteInfo = {0};
		LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
		LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_INVOKEIDLIST;
		LExecuteInfo.lpVerb = "properties";
		LExecuteInfo.lpFile = LFile.c_str();
		LExecuteInfo.nShow = SW_SHOWNORMAL;
		if(!ShellExecuteEx(&LExecuteInfo)) {
			throw EAppError(AnsiString().sprintf(
				"The program could not show the properties of the file \"%s\".",
				LFile.c_str()));
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanShowProperties()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SetPriorityToPrimary()
{
	if(!CanSetPriorityToPrimary()) {
		HandleFeatureUnavailable();
	}
	SetPriorityTo(spPrimary);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSetPriorityToPrimary()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SetPriorityToSecondary()
{
	if(!CanSetPriorityToSecondary()) {
		HandleFeatureUnavailable();
	}
	SetPriorityTo(spSecondary);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanSetPriorityToSecondary()
{
	return EntriesSelection->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::RemoveAllUnique()
{
	if(!CanRemoveAllUnique()) {
		HandleFeatureUnavailable();
	}
	Delete(Entries->FindAllUnique());
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanRemoveAllUnique()
{
	return Entries->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::DropFiles(const TStringDynArray& AFiles)
{
	AddFiles(AFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TScanEntriesCommands::CanDropFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		AddRegularFilesDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanEntriesCommands::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_scanentries
//---------------------------------------------------------------------------

