//---------------------------------------------------------------------------
#ifndef App_ScanEntriesH
#define App_ScanEntriesH
//---------------------------------------------------------------------------
#include <FileCtrl.hpp>
#include "App_Scan.h"
#include "Moa_Lists.h"
namespace Comctrls {class TListView;} using namespace Comctrls;
namespace Dialogs {class TOpenDialog;} using namespace Dialogs;
namespace Moa_clipboard {class TClipboardWatcher;} using namespace Moa_clipboard;
namespace Moa_shellfiles {class TFileTypeCache;} using namespace Moa_shellfiles;
//---------------------------------------------------------------------------
namespace App_scanentries {
//---------------------------------------------------------------------------
class TScanEntry : public TObject
{
public:
	static const DWORD AttributeMask;
private:
	static TFileTypeCache* __fastcall GetFileTypeCache();
	AnsiString FPath;
	AnsiString FDirectory;
	AnsiString FName;
	AnsiString FBaseName;
	AnsiString FExtension;
	DWORDLONG FSize;
	AnsiString FType;
	FILETIME FCreationTime;
	FILETIME FLastWriteTime;
	FILETIME FLastAccessTime;
	DWORD FAttributes;
	AnsiString FHash;
	TScanPriority FPriority;
	int FGroup;
	void __fastcall Initialize(const AnsiString& AFile, const WIN32_FIND_DATA& AFileData);
	void __fastcall Uninitialize();
	void __fastcall SetGroup(int AGroup);
protected:
public:
	__fastcall TScanEntry(const AnsiString& AFile, const WIN32_FIND_DATA& AFileData);
	virtual __fastcall ~TScanEntry();
	virtual TScanRecord __fastcall ToRecord();
	__property AnsiString Path = {read=FPath};
	__property AnsiString Directory = {read=FDirectory};
	__property AnsiString Name = {read=FName};
	__property AnsiString BaseName = {read=FBaseName};
	__property AnsiString Extension = {read=FExtension};
	__property DWORDLONG Size = {read=FSize};
	__property AnsiString Type = {read=FType};
	__property FILETIME CreationTime = {read=FCreationTime};
	__property FILETIME LastWriteTime = {read=FLastWriteTime};
	__property FILETIME LastAccessTime = {read=FLastAccessTime};
	__property DWORD Attributes = {read=FAttributes};
	__property AnsiString Hash = {read=FHash, write=FHash};
	__property TScanPriority Priority = {read=FPriority, write=FPriority};
	__property int Group = {read=FGroup, write=SetGroup};
};
//---------------------------------------------------------------------------
class TScanEntryProducer : public TObject
{
private:
	TStrings* FFiles;
	TStrings* FDirectories;
	TStrings* FHistory;
	TObjectList* FList;
	void __fastcall Initialize(const TStringDynArray& AFiles);
	void __fastcall Uninitialize();
	void __fastcall ProduceFromFile(const AnsiString& AFile);
	void __fastcall ProduceFromDirectory(const AnsiString& ADirectory);
	DynamicArray<TScanEntry*> __fastcall GetValues();
	__property TStrings* Files = {read=FFiles};
	__property TStrings* Directories = {read=FDirectories};
	__property TStrings* History = {read=FHistory};
	__property TObjectList* List = {read=FList};
protected:
public:
	__fastcall TScanEntryProducer(const TStringDynArray& AFiles);
	virtual __fastcall ~TScanEntryProducer();
	virtual bool __fastcall Produce();
	virtual void __fastcall Release();
	__property DynamicArray<TScanEntry*> Values = {read=GetValues};
};
//---------------------------------------------------------------------------
class TScanEntryDescriber
{
public:
	static int ImageIndex_Primary;
	static int ImageIndex_Secondary;
	static int StateIndex_NoResult;
	static int StateIndex_Unique;
	static int StateIndex_Duplicate;
private:
	static int __fastcall EvaluatePriority(TScanPriority APriority);
	static int __fastcall EvaluateGroup(int AGroup);
protected:
public:
	static AnsiString __fastcall DescribeByPath(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByDirectory(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByName(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByBaseName(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByExtension(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeBySize(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByType(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByCreationTime(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByLastWriteTime(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByLastAccessTime(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByAttributes(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByHash(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByPriority(TScanEntry* AEntry);
	static AnsiString __fastcall DescribeByGroup(TScanEntry* AEntry);
	static int __fastcall DetermineImageIndex(TScanEntry* AEntry);
	static int __fastcall DetermineStateIndex(TScanEntry* AEntry);
	static int __fastcall CompareByPath(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByDirectory(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByName(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByBaseName(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByExtension(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareBySize(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByType(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByCreationTime(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByLastWriteTime(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByLastAccessTime(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByAttributes(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByHash(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByPriority(TScanEntry* AEntry1, TScanEntry* AEntry2);
	static int __fastcall CompareByGroup(TScanEntry* AEntry1, TScanEntry* AEntry2);
};
//---------------------------------------------------------------------------
class TScanEntries : public TModel
{
private:
	TObjectList* FList;
	TListIndexesEvent FOnInsertItems;
	TListIndexesEvent FOnDeleteItems;
	TNotifyEvent FOnClear;
	TListReorderEvent FOnReorder;
	TListIndexEvent FOnItemChange;
	static int __fastcall SortCompareByPath(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByDirectory(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByName(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByBaseName(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByExtension(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareBySize(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByType(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByCreationTime(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByLastWriteTime(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByLastAccessTime(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByAttributes(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByHash(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByPriority(int AIndex1, int AIndex2, void* AData);
	static int __fastcall SortCompareByGroup(int AIndex1, int AIndex2, void* AData);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TScanEntry* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	__property TObjectList* List = {read=FList};
protected:
	virtual void __fastcall DoInsertItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoDeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall DoClear();
	virtual void __fastcall DoReorder(const TListReorderRecords& ARecords);
	virtual void __fastcall DoItemChange(int AIndex);
public:
	__fastcall TScanEntries();
	virtual __fastcall ~TScanEntries();
	virtual void __fastcall AddItems(const DynamicArray<TScanEntry*>& AItems);
	virtual void __fastcall DeleteItems(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Clear();
	virtual void __fastcall SortByPath();
	virtual void __fastcall SortByDirectory();
	virtual void __fastcall SortByName();
	virtual void __fastcall SortByBaseName();
	virtual void __fastcall SortByExtension();
	virtual void __fastcall SortBySize();
	virtual void __fastcall SortByType();
	virtual void __fastcall SortByCreationTime();
	virtual void __fastcall SortByLastWriteTime();
	virtual void __fastcall SortByLastAccessTime();
	virtual void __fastcall SortByAttributes();
	virtual void __fastcall SortByHash();
	virtual void __fastcall SortByPriority();
	virtual void __fastcall SortByGroup();
	virtual void __fastcall ReverseOrder();
	virtual void __fastcall MoveUp(const TIntegerDynArray& AIndexes);
	virtual void __fastcall MoveDown(const TIntegerDynArray& AIndexes);
	virtual void __fastcall Reorder(const TListReorderRecords& ARecords);
	virtual int __fastcall Find(TScanEntry* AItem);
	virtual TIntegerDynArray __fastcall FindAllContained(const DynamicArray<TScanEntry*>& AItems);
	virtual TIntegerDynArray __fastcall FindAllDuplicate();
	virtual TIntegerDynArray __fastcall FindAllUnique();
	virtual int __fastcall CountDuplicate();
	virtual int __fastcall CountUnique();
	virtual int __fastcall CountGroups();
	virtual int __fastcall CountByPriority(TScanPriority APriority);
	virtual int __fastcall CountByGroup(int AGroup);
	virtual void __fastcall ItemChange(int AIndex);
	__property TScanEntry* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property TListIndexesEvent OnInsertItems = {read=FOnInsertItems, write=FOnInsertItems};
	__property TListIndexesEvent OnDeleteItems = {read=FOnDeleteItems, write=FOnDeleteItems};
	__property TNotifyEvent OnClear = {read=FOnClear, write=FOnClear};
	__property TListReorderEvent OnReorder = {read=FOnReorder, write=FOnReorder};
	__property TListIndexEvent OnItemChange = {read=FOnItemChange, write=FOnItemChange};
};
//---------------------------------------------------------------------------
class TScanEntriesView : public TView
{
private:
	TScanEntries* FEntries;
	TListView* FListView;
	__property TScanEntries* Entries = {read=FEntries};
	__property TListView* ListView = {read=FListView};
protected:
public:
	__fastcall TScanEntriesView(TScanEntries* AEntries, TListView* AListView);
	virtual __fastcall ~TScanEntriesView();
	virtual void __fastcall HandleInsertItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleDeleteItems(TObject* ASender, const TIntegerDynArray& AIndexes);
	virtual void __fastcall HandleClear(TObject* ASender);
	virtual void __fastcall HandleReorder(TObject* ASender, const TListReorderRecords& ARecords);
	virtual void __fastcall HandleItemChange(TObject* ASender, int AIndex);
};
//---------------------------------------------------------------------------
class TScanEntriesSelection : public TModel
{
private:
	TScanEntries* FEntries;
	TListView* FListView;
	bool __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	int __fastcall GetSelectionCount();
	TIntegerDynArray __fastcall GetSelectionIndexes();
	void __fastcall SetSelectionIndexes(const TIntegerDynArray& AIndexes);
	__property TScanEntries* Entries = {read=FEntries};
	__property TListView* ListView = {read=FListView};
protected:
public:
	__fastcall TScanEntriesSelection(TScanEntries* AEntries, TListView* AListView);
	virtual __fastcall ~TScanEntriesSelection();
	virtual void __fastcall SelectAll();
	virtual void __fastcall ReverseSelection();
	__property bool Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property int SelectionCount = {read=GetSelectionCount};
	__property TIntegerDynArray SelectionIndexes = {read=GetSelectionIndexes, write=SetSelectionIndexes};
};
//---------------------------------------------------------------------------
class TScanEntriesCommands : public TCommands
{
private:
	TScanEntries* FEntries;
	TScanEntriesSelection* FEntriesSelection;
	TClipboardWatcher* FClipboardWatcher;
	TOpenDialog* FAddRegularFilesDialog;
	TNotifyEvent FOnChange;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall AddFiles(const TStringDynArray& LFiles);
	void __fastcall Delete(const TIntegerDynArray& AIndexes);
	void __fastcall SetPriorityTo(TScanPriority APriority);
	void __fastcall ClipboardWatcherChange(TObject* ASender);
	__property TScanEntries* Entries = {read=FEntries};
	__property TScanEntriesSelection* EntriesSelection = {read=FEntriesSelection};
	__property TClipboardWatcher* ClipboardWatcher = {read=FClipboardWatcher};
	__property TOpenDialog* AddRegularFilesDialog = {read=FAddRegularFilesDialog};
protected:
	virtual void __fastcall DoChange();
public:
	__fastcall TScanEntriesCommands(TScanEntries* AEntries, TScanEntriesSelection* AEntriesSelection);
	virtual __fastcall ~TScanEntriesCommands();
	virtual void __fastcall New();
	virtual bool __fastcall CanNew();
	virtual void __fastcall AddRegularFiles();
	virtual bool __fastcall CanAddRegularFiles();
	virtual void __fastcall AddDirectoryEntries();
	virtual bool __fastcall CanAddDirectoryEntries();
	virtual void __fastcall Cut();
	virtual bool __fastcall CanCut();
	virtual void __fastcall Copy();
	virtual bool __fastcall CanCopy();
	virtual void __fastcall Paste();
	virtual bool __fastcall CanPaste();
	virtual void __fastcall Delete();
	virtual bool __fastcall CanDelete();
	virtual void __fastcall SelectAll();
	virtual bool __fastcall CanSelectAll();
	virtual void __fastcall ReverseSelection();
	virtual bool __fastcall CanReverseSelection();
	virtual void __fastcall SelectFirstInGroup();
	virtual bool __fastcall CanSelectFirstInGroup();
	virtual void __fastcall SortByPath();
	virtual bool __fastcall CanSortByPath();
	virtual void __fastcall SortByDirectory();
	virtual bool __fastcall CanSortByDirectory();
	virtual void __fastcall SortByName();
	virtual bool __fastcall CanSortByName();
	virtual void __fastcall SortByBaseName();
	virtual bool __fastcall CanSortByBaseName();
	virtual void __fastcall SortByExtension();
	virtual bool __fastcall CanSortByExtension();
	virtual void __fastcall SortBySize();
	virtual bool __fastcall CanSortBySize();
	virtual void __fastcall SortByType();
	virtual bool __fastcall CanSortByType();
	virtual void __fastcall SortByCreationTime();
	virtual bool __fastcall CanSortByCreationTime();
	virtual void __fastcall SortByLastWriteTime();
	virtual bool __fastcall CanSortByLastWriteTime();
	virtual void __fastcall SortByLastAccessTime();
	virtual bool __fastcall CanSortByLastAccessTime();
	virtual void __fastcall SortByAttributes();
	virtual bool __fastcall CanSortByAttributes();
	virtual void __fastcall SortByHash();
	virtual bool __fastcall CanSortByHash();
	virtual void __fastcall SortByPriority();
	virtual bool __fastcall CanSortByPriority();
	virtual void __fastcall SortByGroup();
	virtual bool __fastcall CanSortByGroup();
	virtual void __fastcall ReverseOrder();
	virtual bool __fastcall CanReverseOrder();
	virtual void __fastcall MoveUp();
	virtual bool __fastcall CanMoveUp();
	virtual void __fastcall MoveDown();
	virtual bool __fastcall CanMoveDown();
	virtual void __fastcall Open();
	virtual bool __fastcall CanOpen();
	virtual void __fastcall ExploreDirectory();
	virtual bool __fastcall CanExploreDirectory();
	virtual void __fastcall MoveToRecycleBin();
	virtual bool __fastcall CanMoveToRecycleBin();
	virtual void __fastcall ShowTotalSize();
	virtual bool __fastcall CanShowTotalSize();
	virtual void __fastcall ShowProperties();
	virtual bool __fastcall CanShowProperties();
	virtual void __fastcall SetPriorityToPrimary();
	virtual bool __fastcall CanSetPriorityToPrimary();
	virtual void __fastcall SetPriorityToSecondary();
	virtual bool __fastcall CanSetPriorityToSecondary();
	virtual void __fastcall RemoveAllUnique();
	virtual bool __fastcall CanRemoveAllUnique();
	virtual void __fastcall DropFiles(const TStringDynArray& AFiles);
	virtual bool __fastcall CanDropFiles();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
};
//---------------------------------------------------------------------------
} // End namespace App_scanentries
using namespace App_scanentries;
//---------------------------------------------------------------------------
#endif

