//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_ScanTasks.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_scantasks {
//---------------------------------------------------------------------------
TScanCommand* __fastcall TScanTask::CreateCommand()
{
	TCreateCommandArguments LArguments(NULL);
	try {
		Synchronize(&SynchronizedCreateCommand, &LArguments);
		if(Canceled) {
			Abort();
		}
	}
	catch(...) {
		delete LArguments.Command;
		LArguments.Command = NULL;
		throw;
	}
	return LArguments.Command;
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::MatcherContinue(TObject* ASender, bool& ACanContinue)
{
	ACanContinue = !Canceled;
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::CommandMatching(TObject* ASender, int AIndex, const TScanRecord& ARecord)
{
	TCommandMatchArguments LArguments(ASender, AIndex, ARecord);
	Synchronize(&SynchronizedCommandMatching, &LArguments);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::CommandMatch(TObject* ASender, int AIndex, const TScanRecord& ARecord)
{
	TCommandMatchArguments LArguments(ASender, AIndex, ARecord);
	Synchronize(&SynchronizedCommandMatch, &LArguments);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::CommandGroup(TObject* ASender, int AIndex, const TScanRecord& ARecord)
{
	TCommandGroupArguments LArguments(ASender, AIndex, ARecord);
	Synchronize(&SynchronizedCommandGroup, &LArguments);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::CommandProgress(TObject* ASender, int ACurrent, int AAll)
{
	DoProgress(ACurrent, AAll);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::CommandContinue(TObject* ASender, bool& ACanContinue)
{
	ACanContinue = !Canceled;
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::SynchronizedCreateCommand(void* AData)
{
	try {
		TAutoPointer<TScanCommand> LCommand;
		{
			DynamicArray<TScanRecord> LRecords;
			LRecords.Length = Document->Entries->Count;
			for(int LI = 0; LI < LRecords.Length; ++LI) {
				LRecords[LI] = Document->Entries->Items[LI]->ToRecord();
			}

			// Add matchers from the lightest to the heaviest
			TAutoPointer<TScanMatchers> LMatchers = new TScanMatchers();
			if(Document->Aspects.Contains(saAttributes)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanAttributesMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saSize)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanSizeMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saCreationTime)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanCreationTimeMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saLastWriteTime)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanLastWriteTimeMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saLastAccessTime)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanLastAccessTimeMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saBaseName)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanBaseNameMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saExtension)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanExtensionMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saName)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanNameMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saDirectory)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanDirectoryMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saType)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanTypeMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saHash)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanHashMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}
			if(Document->Aspects.Contains(saBinaries)) {
				TAutoPointer<TScanMatcher> LMatcher = new TScanBinariesMatcher();
				LMatcher->OnContinue = MatcherContinue;
				LMatchers->Add(LMatcher);
				LMatcher.Release();
			}

			LCommand = new TScanCommand(LRecords, LMatchers);
			LMatchers.Release();
		}
		LCommand->OnMatching = CommandMatching;
		LCommand->OnMatch = CommandMatch;
		LCommand->OnGroup = CommandGroup;
		LCommand->OnProgress = CommandProgress;
		LCommand->OnContinue = CommandContinue;
		TCreateCommandArguments& LArguments = *static_cast<TCreateCommandArguments*>(AData);
		LArguments.Command = LCommand.Release();
	}
	catch(...) {
		Cancel();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::SynchronizedCommandMatching(void* AData)
{
	TCommandMatchArguments& LArguments = *static_cast<TCommandMatchArguments*>(AData);
	DoStatus(AnsiString().sprintf("Matching \"%s\"...", LArguments.Record.Name.c_str()));
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::SynchronizedCommandMatch(void* AData)
{
	TCommandMatchArguments& LArguments = *static_cast<TCommandMatchArguments*>(AData);
	DoStatus(AnsiString().sprintf("Matched \"%s\"", LArguments.Record.Name.c_str()));
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::SynchronizedCommandGroup(void* AData)
{
	TCommandGroupArguments& LArguments = *static_cast<TCommandGroupArguments*>(AData);
	Document->Entries->Items[LArguments.Index]->Hash = LArguments.Record.Hash;
	Document->Entries->Items[LArguments.Index]->Group = LArguments.Record.Group;
	Document->Entries->ItemChange(LArguments.Index);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::SynchronizedDoStatus(void* AData)
{
	TDoStatusArguments& LArguments = *static_cast<TDoStatusArguments*>(AData);
	OnStatus(this, LArguments.Message);
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::Execute()
{
	TAutoPointer<TScanCommand> LCommand = CreateCommand();
	LCommand->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::DoStatus(const AnsiString& AMessage)
{
	if(OnStatus != NULL) {
		TDoStatusArguments LArguments(AMessage);
		Synchronize(&SynchronizedDoStatus, &LArguments);
	}
}
//---------------------------------------------------------------------------
__fastcall TScanTask::TScanTask(TDocument* ADocument) :
	TTask(),
	FDocument(ADocument)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanTask::TScanTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TScanTask::HandleCancel(TObject* ASender)
{
	Cancel();
}
//---------------------------------------------------------------------------
} // End namespace App_scantasks
//---------------------------------------------------------------------------

