//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <ComCtrls.hpp>
#include <Forms.hpp>
#include <IniFiles.hpp>
#include "App_View.h"
#include "Moa_Forms.h"
#include "Moa_Sets.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_view {
//---------------------------------------------------------------------------
TScanFields __fastcall TViewSettings::GetFields()
{
	return FFields;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettings::SetFields(const TScanFields& AFields)
{
	if(AFields == Fields) {
		return;
	}
	FFields = AFields;
	DoFieldsChange();
}
//---------------------------------------------------------------------------
void __fastcall TViewSettings::DoFieldsChange()
{
	if(OnFieldsChange != NULL) {
		OnFieldsChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TViewSettings::TViewSettings(TForm* AForm) :
	TModel(),
	FForm(AForm),
	FFields(TScanFields() << sfName << sfDirectory << sfPriority << sfGroup),
	FOnFieldsChange(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TViewSettings::~TViewSettings()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TViewSettings::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".Form";
	{
		TRect LBounds = TFormUtilities::GetNormalBounds(Form);
		LBounds.left = ASettings->ReadInteger(
			LSection,
			"Left",
			LBounds.left);
		LBounds.top = ASettings->ReadInteger(
			LSection,
			"Top",
			LBounds.top);
		LBounds.right = ASettings->ReadInteger(
			LSection,
			"Right",
			LBounds.right);
		LBounds.bottom = ASettings->ReadInteger(
			LSection,
			"Bottom",
			LBounds.bottom);
		TFormUtilities::SetNormalBounds(Form, LBounds);
	}
	LSection = ASection;
	{
		Fields = TSetUtilities<TScanField, sfPath, sfGroup>::FromString(ASettings->ReadString(
			LSection,
			"Fields",
			TSetUtilities<TScanField, sfPath, sfGroup>::ToString(Fields)));
	}
}
//---------------------------------------------------------------------------
void __fastcall TViewSettings::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".Form";
	{
		TRect LBounds = TFormUtilities::GetNormalBounds(Form);
		ASettings->WriteInteger(
			LSection,
			"Left",
			LBounds.left);
		ASettings->WriteInteger(
			LSection,
			"Top",
			LBounds.top);
		ASettings->WriteInteger(
			LSection,
			"Right",
			LBounds.right);
		ASettings->WriteInteger(
			LSection,
			"Bottom",
			LBounds.bottom);
		TFormUtilities::SetNormalBounds(Form, LBounds);
	}
	LSection = ASection;
	{
		ASettings->WriteString(
			LSection,
			"Fields",
			TSetUtilities<TScanField, sfPath, sfGroup>::ToString(Fields));
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TListColumn* __fastcall TViewSettingsView::AddListColumn(TListView* AListView, const AnsiString& ACaption, TAlignment AAlignment, int AWidth)
{
	TListColumn* LListColumn = AListView->Columns->Add();
	LListColumn->Caption = ACaption;
	LListColumn->Alignment = AAlignment;
	LListColumn->Width = AWidth;
	return LListColumn;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::Initialize()
{
	try {
		for(int LI = 0; LI < FieldCount; ++LI) {
			FieldWidths[LI] = 100;
		}
		FieldWidths[2] = 200;
		Settings->OnFieldsChange = HandleFieldsChange;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::UpdateFieldWidths()
{
	int LIndex = -1;
	if(Fields.Contains(sfName)) {
		FieldWidths[2] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfPath)) {
		FieldWidths[0] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfDirectory)) {
		FieldWidths[1] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfBaseName)) {
		FieldWidths[3] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfExtension)) {
		FieldWidths[4] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfSize)) {
		FieldWidths[5] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfType)) {
		FieldWidths[6] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfCreationTime)) {
		FieldWidths[7] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfLastWriteTime)) {
		FieldWidths[8] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfLastAccessTime)) {
		FieldWidths[9] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfAttributes)) {
		FieldWidths[10] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfHash)) {
		FieldWidths[11] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfPriority)) {
		FieldWidths[12] = ListView->Columns->Items[++LIndex]->Width;
	}
	if(Fields.Contains(sfGroup)) {
		FieldWidths[13] = ListView->Columns->Items[++LIndex]->Width;
	}
	FFields = Settings->Fields;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::UpdateListView()
{
	ListView->Items->BeginUpdate();
	try {
		ListView->Columns->BeginUpdate();
		try {
			ListView->Columns->Clear();
			if(Settings->Fields.Contains(sfName)) {
				AddListColumn(ListView, "Name", taLeftJustify, FieldWidths[2]);
			}
			if(Settings->Fields.Contains(sfPath)) {
				AddListColumn(ListView, "Path", taLeftJustify, FieldWidths[0]);
			}
			if(Settings->Fields.Contains(sfDirectory)) {
				AddListColumn(ListView, "Directory", taLeftJustify, FieldWidths[1]);
			}
			if(Settings->Fields.Contains(sfBaseName)) {
				AddListColumn(ListView, "Base name", taLeftJustify, FieldWidths[3]);
			}
			if(Settings->Fields.Contains(sfExtension)) {
				AddListColumn(ListView, "Extension", taLeftJustify, FieldWidths[4]);
			}
			if(Settings->Fields.Contains(sfSize)) {
				AddListColumn(ListView, "Size", taRightJustify, FieldWidths[5]);
			}
			if(Settings->Fields.Contains(sfType)) {
				AddListColumn(ListView, "Type", taLeftJustify, FieldWidths[6]);
			}
			if(Settings->Fields.Contains(sfCreationTime)) {
				AddListColumn(ListView, "Creation time", taLeftJustify, FieldWidths[7]);
			}
			if(Settings->Fields.Contains(sfLastWriteTime)) {
				AddListColumn(ListView, "Last write time", taLeftJustify, FieldWidths[7]);
			}
			if(Settings->Fields.Contains(sfLastAccessTime)) {
				AddListColumn(ListView, "Last access time", taLeftJustify, FieldWidths[8]);
			}
			if(Settings->Fields.Contains(sfAttributes)) {
				AddListColumn(ListView, "Attributes", taLeftJustify, FieldWidths[10]);
			}
			if(Settings->Fields.Contains(sfHash)) {
				AddListColumn(ListView, "Hash", taLeftJustify, FieldWidths[11]);
			}
			if(Settings->Fields.Contains(sfPriority)) {
				AddListColumn(ListView, "Priority", taLeftJustify, FieldWidths[12]);
			}
			if(Settings->Fields.Contains(sfGroup)) {
				AddListColumn(ListView, "Group", taRightJustify, FieldWidths[13]);
			}
		}
		catch(...) {
			ListView->Columns->EndUpdate();
			throw;
		}
		ListView->Columns->EndUpdate();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
TScanFields __fastcall TViewSettingsView::GetFields()
{
	return FFields;
}
//---------------------------------------------------------------------------
int __fastcall TViewSettingsView::GetFieldWidth(int AIndex)
{
	return FFieldWidths[AIndex];
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::SetFieldWidth(int AIndex, int AFieldWidth)
{
	FFieldWidths[AIndex] = AFieldWidth;
}
//---------------------------------------------------------------------------
__fastcall TViewSettingsView::TViewSettingsView(TViewSettings* ASettings, TListView* AListView) :
	TView(),
	FSettings(ASettings),
	FListView(AListView),
	FFields(),
	FFieldWidths()
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TViewSettingsView::~TViewSettingsView()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::HandleFieldsChange(TObject* ASender)
{
	UpdateFieldWidths();
	UpdateListView();
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		UpdateFieldWidths();
		for(int LI = 0; LI < FieldCount; ++LI) {
			FieldWidths[LI] = ASettings->ReadInteger(
				LSection,
				AnsiString().sprintf("FieldWidths[%d]", LI),
				FieldWidths[LI]);
		}
		UpdateListView();
	}
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsView::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		UpdateFieldWidths();
		for(int LI = 0; LI < FieldCount; ++LI) {
			ASettings->WriteInteger(
				LSection,
				AnsiString().sprintf("FieldWidths[%d]", LI),
				FieldWidths[LI]);
		}
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowField(TScanField AField)
{
	if(Settings->Fields.Contains(AField)) {
		Settings->Fields = Settings->Fields >> AField;
	}
	else {
		Settings->Fields = Settings->Fields << AField;
	}
}
//---------------------------------------------------------------------------
__fastcall TViewSettingsCommands::TViewSettingsCommands(TViewSettings* ASettings) :
	TCommands(),
	FSettings(ASettings)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TViewSettingsCommands::~TViewSettingsCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowPathField()
{
	if(!CanSwitchShowPathField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfPath);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowPathField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowDirectoryField()
{
	if(!CanSwitchShowDirectoryField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfDirectory);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowDirectoryField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowBaseNameField()
{
	if(!CanSwitchShowBaseNameField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfBaseName);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowBaseNameField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowExtensionField()
{
	if(!CanSwitchShowExtensionField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfExtension);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowExtensionField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowSizeField()
{
	if(!CanSwitchShowSizeField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfSize);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowSizeField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowTypeField()
{
	if(!CanSwitchShowTypeField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfType);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowTypeField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowCreationTimeField()
{
	if(!CanSwitchShowCreationTimeField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfCreationTime);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowCreationTimeField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowLastWriteTimeField()
{
	if(!CanSwitchShowLastWriteTimeField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfLastWriteTime);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowLastWriteTimeField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowLastAccessTimeField()
{
	if(!CanSwitchShowLastAccessTimeField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfLastAccessTime);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowLastAccessTimeField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowAttributesField()
{
	if(!CanSwitchShowAttributesField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfAttributes);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowAttributesField()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewSettingsCommands::SwitchShowHashField()
{
	if(!CanSwitchShowHashField()) {
		HandleFeatureUnavailable();
	}
	SwitchShowField(sfHash);
}
//---------------------------------------------------------------------------
bool __fastcall TViewSettingsCommands::CanSwitchShowHashField()
{
	return true;
}
//---------------------------------------------------------------------------
} // End namespace App_view
//---------------------------------------------------------------------------

