//---------------------------------------------------------------------------
#ifndef App_ViewH
#define App_ViewH
//---------------------------------------------------------------------------
#include "App_Common.h"
namespace Comctrls {class TListView;} using namespace Comctrls;
namespace Forms {class TForm;} using namespace Forms;
//---------------------------------------------------------------------------
namespace App_view {
//---------------------------------------------------------------------------
enum TScanField
{
	sfPath,
	sfDirectory,
	sfName,
	sfBaseName,
	sfExtension,
	sfSize,
	sfType,
	sfCreationTime,
	sfLastWriteTime,
	sfLastAccessTime,
	sfAttributes,
	sfHash,
	sfPriority,
	sfGroup,
};
typedef Set<TScanField, sfPath, sfGroup> TScanFields;
//---------------------------------------------------------------------------
class TViewSettings : public TModel
{
private:
	TForm* FForm;
	TScanFields FFields;
	TNotifyEvent FOnFieldsChange;
	TScanFields __fastcall GetFields();
	void __fastcall SetFields(const TScanFields& AFields);
	__property TForm* Form = {read=FForm};
protected:
	virtual void __fastcall DoFieldsChange();
public:
	__fastcall TViewSettings(TForm* AForm);
	virtual __fastcall ~TViewSettings();
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	__property TScanFields Fields = {read=GetFields, write=SetFields};
	__property TNotifyEvent OnFieldsChange = {read=FOnFieldsChange, write=FOnFieldsChange};
};
//---------------------------------------------------------------------------
class TViewSettingsView : public TView
{
private:
	static const int FieldCount = static_cast<int>(sfGroup) - static_cast<int>(sfPath) + 1;
	TViewSettings* FSettings;
	TListView* FListView;
	TScanFields FFields;
	int FFieldWidths[FieldCount];
	static TListColumn* __fastcall AddListColumn(TListView* AListView, const AnsiString& ACaption, TAlignment AAlignment, int AWidth);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall UpdateFieldWidths();
	void __fastcall UpdateListView();
	TScanFields __fastcall GetFields();
	int __fastcall GetFieldWidth(int AIndex);
	void __fastcall SetFieldWidth(int AIndex, int AFieldWidth);
	__property TViewSettings* Settings = {read=FSettings};
	__property TListView* ListView = {read=FListView};
	__property TScanFields Fields = {read=GetFields};
	__property int FieldWidths[int AIndex] = {read=GetFieldWidth, write=SetFieldWidth};
protected:
public:
	__fastcall TViewSettingsView(TViewSettings* ASettings, TListView* AListView);
	virtual __fastcall ~TViewSettingsView();
	virtual void __fastcall HandleFieldsChange(TObject* ASender);
	virtual void __fastcall LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection);
};
//---------------------------------------------------------------------------
class TViewSettingsCommands : public TCommands
{
private:
	TViewSettings* FSettings;
	void __fastcall SwitchShowField(TScanField AField);
	__property TViewSettings* Settings = {read=FSettings};
protected:
public:
	__fastcall TViewSettingsCommands(TViewSettings* ASettings);
	virtual __fastcall ~TViewSettingsCommands();

	virtual void __fastcall SwitchShowPathField();
	virtual bool __fastcall CanSwitchShowPathField();

	virtual void __fastcall SwitchShowDirectoryField();
	virtual bool __fastcall CanSwitchShowDirectoryField();

	virtual void __fastcall SwitchShowBaseNameField();
	virtual bool __fastcall CanSwitchShowBaseNameField();
	virtual void __fastcall SwitchShowExtensionField();
	virtual bool __fastcall CanSwitchShowExtensionField();
	virtual void __fastcall SwitchShowSizeField();
	virtual bool __fastcall CanSwitchShowSizeField();
	virtual void __fastcall SwitchShowTypeField();
	virtual bool __fastcall CanSwitchShowTypeField();
	virtual void __fastcall SwitchShowCreationTimeField();
	virtual bool __fastcall CanSwitchShowCreationTimeField();
	virtual void __fastcall SwitchShowLastWriteTimeField();
	virtual bool __fastcall CanSwitchShowLastWriteTimeField();
	virtual void __fastcall SwitchShowLastAccessTimeField();
	virtual bool __fastcall CanSwitchShowLastAccessTimeField();
	virtual void __fastcall SwitchShowAttributesField();
	virtual bool __fastcall CanSwitchShowAttributesField();
	virtual void __fastcall SwitchShowHashField();
	virtual bool __fastcall CanSwitchShowHashField();
};
//---------------------------------------------------------------------------
} // End namespace App_view
using namespace App_view;
//---------------------------------------------------------------------------
#endif

