//---------------------------------------------------------------------------
#ifndef Moa_AutoPointersH
#define Moa_AutoPointersH
//---------------------------------------------------------------------------
// Version: 1.0.1
// Date: 2012-06-24 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_autopointers {
//---------------------------------------------------------------------------
template<typename T>
class TCustomAutoPointer
{
private:
	void* __fastcall operator new(std::size_t);
	void __fastcall operator delete(void* APointer) {}
protected:
	T* FPointer;
	__fastcall TCustomAutoPointer() :
		FPointer(NULL)
	{
		// Do nothing
	}
	__fastcall TCustomAutoPointer(T* APointer) :
		FPointer(APointer)
	{
		// Do nothing
	}
public:
	T& __fastcall operator *() const
	{
		return *FPointer;
	}
	__fastcall operator T* () const
	{
		return FPointer;
	}
	bool __fastcall operator !() const
	{
		return FPointer == NULL;
	}
	T* __fastcall operator->() const
	{
		return FPointer;
	}
	T** __fastcall operator &()
	{
		return &FPointer;
	}
	bool __fastcall operator ==(T* AOther) const
	{
		return FPointer == AOther;
	}
	bool __fastcall operator ==(const TCustomAutoPointer<T> &AOther) const
	{
		return FPointer == AOther.FPointer;
	}
	bool __fastcall operator !=(T* AOther) const
	{
		return FPointer != AOther;
	}
	bool __fastcall operator !=(const TCustomAutoPointer<T> &AOther) const
	{
		return FPointer != AOther.FPointer;
	}
	T* __fastcall Release()
	{
		return Reset(NULL);
	}
	T* __fastcall Reset(T* APointer)
	{
		T* LOldPointer = FPointer;
		FPointer = APointer;
		return LOldPointer;
	}
	__property T* Pointer = {read=FPointer};
};
//---------------------------------------------------------------------------
template<typename T>
class TAutoPointer : public TCustomAutoPointer<T>
{
private:
protected:
public:
	__fastcall TAutoPointer() :
		TCustomAutoPointer<T>()
	{
		// Do nothing
	}
	__fastcall TAutoPointer(T* AOther) :
		TCustomAutoPointer<T>(AOther)
	{
		// Do nothing
	}
	__fastcall TAutoPointer(TAutoPointer<T>& AOther) :
		TCustomAutoPointer<T>(AOther.Release())
	{
		// Do nothing
	}
	__fastcall ~TAutoPointer()
	{
		Clear(NULL);
	}
	TAutoPointer& __fastcall operator =(TAutoPointer<T>& AOther)
	{
		Reset(AOther.Release());
		return *this;
	}
	TAutoPointer& __fastcall operator =(T* AOther)
	{
		Clear(AOther);
		return *this;
	}
	T* __fastcall operator &() const
	{
		return Pointer;
	}
	void __fastcall Clear(T* AOther)
	{
		if(FPointer != NULL) {
			delete FPointer;
		}
		FPointer = AOther;
	}
};
//---------------------------------------------------------------------------
template<typename T>
class TAutoArray : public TCustomAutoPointer<T>
{
public:
	__fastcall TAutoArray() :
		TCustomAutoPointer<T>()
	{
		// Do nothing
	}
	__fastcall TAutoArray(T AOther[]) :
		TCustomAutoPointer<T>(AOther)
	{
		// Do nothing
	}
	__fastcall TAutoArray(TAutoPointer<T>& AOther) :
		TCustomAutoPointer<T>(AOther.Release())
	{
		// Do nothing
	}
	__fastcall ~TAutoArray()
	{
		Clear(0);
	}
	TAutoArray& __fastcall operator =(TAutoArray<T>& AOther)
	{
		Reset(AOther.Release());
		return *this;
	}
	TAutoArray& __fastcall operator =(T AOther[])
	{
		Clear(AOther);
		return *this;
	}
	T& __fastcall operator [](int AIndex) const
	{
		return FPointer[AIndex];
	}
	void __fastcall Clear(T AOther[])
	{
		if(FPointer != NULL) {
			delete[] FPointer;
		}
		FPointer = AOther;
	}
};
//---------------------------------------------------------------------------
} // End namespace Moa_autopointers
using namespace Moa_autopointers;
//---------------------------------------------------------------------------
#endif

